@@ -1,5 +1,5 @@
 /*
-	Auther	Satoshi Yasuda	7m3tjz/ad6gz
+	Auther		Satoshi Yasuda	7m3tjz/ad6gz
 */
 
 #include <stdio.h>
@@ -12,6 +12,7 @@
 #include <signal.h>
 #include "dv_echo.h"
 #include "shm.h"
+#include "holeP.h"
 
 #define	TRUE	1
 #define	FALSE	0
@@ -35,10 +36,10 @@ char	ConnectWait;
 char	FirstHeaderSend;
 int	AckTimeOut;
 int	DSTREQcnt;
-FILE	*in_dat;
 char	SendBack;
 char	Connected;
 char	DVwait;
+char	AckSW;
 unsigned char	RcvPacketIdSave[2];
 unsigned char	SndPacketIdSave[2];
 unsigned int	crc;
@@ -56,6 +57,10 @@ void CloseSocket()
                 close(dstSocket);
                 dstSocket=0;
         }
+	logfile = fopen (LOGFILE, "a");
+	now = time(NULL);
+	fprintf (logfile, "%24.24s dv_echo end\n",ctime(&now));
+	fclose (logfile);
         exit(0);
 }
 
@@ -63,12 +68,15 @@ void	voice_send(void)
 {
 	if (ConnectWait) return;
 
-	if (in_dat == NULL)
+	if (dat == NULL)
 	{
-		in_dat = fopen ("/tmp/dv_echo.bin", "rb");
-		fread ((char *)&send_pkt, 56, 1, in_dat);
+		dat = fopen ("/tmp/dv_echo.bin", "rb");
+		fread ((char *)&send_pkt, 56, 1, dat);
         	if (send_pkt.B_header.seq == 0x80)
         	{
+			Packet_ID = rand();
+			SndPacketIdSave[0] = (Packet_ID >> 8) & 0xff;
+			SndPacketIdSave[1] = Packet_ID & 0xff;
                 	ConSend();
                 	ConnectWait = TRUE;
 			AckTimeOut = 0;
@@ -99,8 +107,8 @@ void	voice_frame_send(void)
                         memcpy ((char *)&RelayPacket.dv, (char *)&send_pkt, 56);
                         RelayPacket.ip = dstIP;
                         RelayPacket.port = dstPort;
-        		RelayPacket.dv.B_header.PacketID[0] = (Packet_ID >> 8) & 0xff;
-        		RelayPacket.dv.B_header.PacketID[1] = Packet_ID & 0xff;
+        		RelayPacket.dv.B_header.PacketID[0] = SndPacketIdSave[0];
+        		RelayPacket.dv.B_header.PacketID[1] = SndPacketIdSave[1];
                         sndAddr.sin_family = PF_INET;
                         sndAddr.sin_port = db_port;
                         sndAddr.sin_addr = dbIP;
@@ -115,8 +123,8 @@ void	voice_frame_send(void)
                 }
                 else
                 {
-                        send_pkt.B_header.PacketID[0] = (Packet_ID >> 8) & 0xff;
-                        send_pkt.B_header.PacketID[1] = Packet_ID & 0xff;
+                        send_pkt.B_header.PacketID[0] = SndPacketIdSave[0];
+                        send_pkt.B_header.PacketID[1] = SndPacketIdSave[1];
                         sndAddr.sin_addr = dstIP;
                         sndAddr.sin_port = dstPort;
                         sndAddr.sin_family = PF_INET;
@@ -141,11 +149,11 @@ void	voice_frame_send(void)
        	gettimeofday (&t2, NULL);
        	if (timercmp (&t2, &t1, >))
        	{
-		if (fread ((char *)&send_pkt, 56, 1, in_dat) <= 0)
+		if (fread ((char *)&send_pkt, 56, 1, dat) <= 0)
 		{
 			SendBack = FALSE;
-			fclose(in_dat);
-			in_dat = NULL;
+			fclose(dat);
+			dat = NULL;
 			RelaySW = FALSE;
 			return;
 		}
@@ -154,8 +162,8 @@ void	voice_frame_send(void)
 			memcpy	((char *)&RelayPacket.dv, (char *)&send_pkt, 27);
 			RelayPacket.ip = dstIP;
 			RelayPacket.port = dstPort;
-                        RelayPacket.dv.B_header.PacketID[0] = (Packet_ID >> 8) & 0xff;
-                        RelayPacket.dv.B_header.PacketID[1] = Packet_ID & 0xff;
+                        RelayPacket.dv.B_header.PacketID[0] = SndPacketIdSave[0];
+                        RelayPacket.dv.B_header.PacketID[1] = SndPacketIdSave[1];
 			sndAddr.sin_family = PF_INET;
 			sndAddr.sin_port = db_port;
 			sndAddr.sin_addr = dbIP;
@@ -164,8 +172,8 @@ void	voice_frame_send(void)
 		}
 		else
 		{
-                        send_pkt.B_header.PacketID[0] = (Packet_ID >> 8) & 0xff;
-                        send_pkt.B_header.PacketID[1] = Packet_ID & 0xff;
+                        send_pkt.B_header.PacketID[0] = SndPacketIdSave[0]; 
+                        send_pkt.B_header.PacketID[1] = SndPacketIdSave[1];;
 			sndAddr.sin_addr = dstIP;
 			sndAddr.sin_port = dstPort;
 			sndAddr.sin_family = PF_INET;
@@ -177,6 +185,11 @@ void	voice_frame_send(void)
 			ConnectWait = FALSE;
 			FirstHeaderSend = FALSE;
 			Connected = FALSE;
+                        SendBack = FALSE;
+                        fclose(dat);
+                        dat = NULL;
+                        RelaySW = FALSE;
+                        return;
 		}
                	timeradd (&t1, &t4, &t3);
                	t1.tv_sec = t3.tv_sec;
@@ -232,7 +245,7 @@ void    ReqSendSender()
         sndAddr.sin_family = PF_INET;
         sndAddr.sin_port = dstPort;
         sndAddr.sin_addr = dstIP;
-        sendto (udpSocket, "\0\0\0\0DSTREQ UDP hole punching", 28, 0,
+        sendto (udpSocket, DSTREQ, 5, 0,
                 (struct sockaddr *)&sndAddr, sizeof(sndAddr));
 }
 
@@ -280,7 +293,8 @@ int main(int argc, const char **argv){
                 {
                         logfile = fopen (LOGFILE, "a");
                         now = time(NULL);
-                        fprintf (logfile, "%24.24s ERROR FIFO file not open /tmp/dv_echo-cmdin\n", ctime(&now));
+                        fprintf (logfile, "%24.24s ERROR FIFO file not open /tmp/dv_echo-cmdin\n", 
+				ctime(&now));
                         fclose (logfile);
                         cmdin = 0;
                 }
@@ -378,11 +392,11 @@ int main(int argc, const char **argv){
 	ConnectWait = FALSE;
 	FirstHeaderSend = FALSE;
 	AckTimeOut = 0;
-	in_dat = NULL;
 	dat = NULL;
 	NakCount = 0;
 	Connected = FALSE;
 	DVwait = FALSE;
+	AckSW = FALSE;
 
 	while (1){
 		readOk = Mask;
@@ -444,26 +458,44 @@ int main(int argc, const char **argv){
                         		{
 						if (!Connected && !memcmp(&SndPacketIdSave, &reqConMsg.ackConnect.PacketID, 2))
 						{
+							logfile = fopen (LOGFILE, "a");
+							now = time(NULL);
+							fprintf (logfile, "%24.24s Receive Ack\n", ctime(&now));
+							fclose (logfile);
                                                		dstPort = reqConMsg.ackConnect.port;
                                                 	dstIP = reqConMsg.ackConnect.ip;
                                         		udpAddr.sin_family = PF_INET;
                                                 	udpAddr.sin_port = dstPort;
                                               		udpAddr.sin_addr = dstIP;
 							respFlags = RespNULL;
-                                                	sendto (udpSocket, "\0\0\0\0SRCREQ UDP hole punching", 28, 0,
+							AckSW = TRUE;
+                                                	sendto (udpSocket, SRCREQ, 5, 0,
                                                        		(struct sockaddr *)&udpAddr, sizeof(udpAddr));
 						}
                         		}
                                         if (reqConMsg.reqType == Nak)
                                         {
-						usleep(500000);
+						logfile = fopen (LOGFILE, "a");
+						now = time(NULL);
+						fprintf (logfile, "%24.24s Receive Busy\n", ctime(&now));
+						fclose(logfile);
+						usleep(200000);	/* wait 200 m Sec. */
 						ConSend();
 						AckTimeOut = 0;
 						NakCount++;
-						if (NakCount > 20)
+						if (NakCount > 10)	/* 2 Sec. */ 	
 						{
-							ConnectWait = FALSE;
-							FirstHeaderSend = FALSE;
+				                        ConnectWait = FALSE;
+                        				FirstHeaderSend = FALSE;
+                        				Connected = FALSE;
+                        				SendBack = FALSE;
+                        				fclose(dat);
+                        				dat = NULL;
+                        				RelaySW = FALSE;
+							logfile = fopen (LOGFILE, "a");
+							now = time(NULL);
+							fprintf (logfile, "%24.24s Give up to echo (wait 2 sec.)\n", ctime(&now));
+							fclose (logfile);
 						}
                                         }
                                         else if (reqConMsg.reqType == ReqConnect)
@@ -492,15 +524,16 @@ int main(int argc, const char **argv){
                 			send (dstSocket, &reqConMsg, sizeof(reqConMsg), 0);
 					Send_Connect = FALSE;
 					Connected = FALSE;
+					AckSW = FALSE;
 				}
 				if (FD_ISSET(udpSocket, &readOk))
 				{
 			                addrlen = sizeof(struct sockaddr_in);
 			                ret=recvfrom (udpSocket, (char *)&send_packet, 56, 0,
                         			(struct sockaddr *)&udpAddr, &addrlen);
-					if (memcmp ((char *)&send_packet, "\0\0\0\0DSTREQ", 10) == 0)
+					if (memcmp ((char *)&send_packet, DSTREQ, 5) == 0)
 					{
-						if (!Connected)
+						if (!Connected && AckSW)
 						{
                                                 	ConnectWait = FALSE;
                                                 	FirstHeaderSend = TRUE;
@@ -510,14 +543,14 @@ int main(int argc, const char **argv){
 							DVwait = FALSE;
 						}
 					}
-					if (memcmp ((char *)&send_packet, "\0\0\0\0SRCREQ", 10) == 0)
+					if (memcmp ((char *)&send_packet, SRCREQ, 5) == 0)
 					{
-                        			sendto (udpSocket, "\0\0\0\0SRCACK UDP hole punching", 28, 0,
+                        			sendto (udpSocket, SRCACK, 5, 0,
                                 			(struct sockaddr *)&udpAddr, sizeof(udpAddr));
 					}
-        				if (memcmp ((char *)&send_packet, "\0\0\0\0SRCACK", 10) == 0)
+        				if (memcmp ((char *)&send_packet, SRCACK, 5) == 0)
         				{
-						if (!Connected)
+						if (!Connected && AckSW)
 						{
 							dstPort = udpAddr.sin_port;
 							dstIP = udpAddr.sin_addr;
@@ -548,9 +581,11 @@ int main(int argc, const char **argv){
 								fclose (dat);
 								dat = NULL;
 							}
+							usleep (700000);
 							SendBack = TRUE;
 							Connected = FALSE;
 							ConnectWait = FALSE;
+							AckSW = FALSE;
 						}
 					}
 				}
